@@ -5,22 +5,25 @@ import sys
 import os
 import datetime
 import platform
+import csv
 
 
 class Query(object):
 
     BASE_URL = 'https://www.kgbase.com/kgbase-query'
-    BULK_UPLOAD_URL = "https://kgbase.com/bulk/upload"
+    BULK_UPLOAD_URL = "https://www.kgbase.com/bulk/upload"
     HEADERS = {
         "Accept": "application/json",
         "Content-Type": "application/json",
-        "User-Agent": "Python API 0.24 / {local_version}".format(local_version=platform.python_version())
+        "User-Agent": "Python API 0.33 / {local_version}".format(local_version=platform.python_version())
     }
 
     def __init__(self, proxies={}, verify=True):
         self._session = requests.session()
+        self._session.trust_env = False
         self._proxies = proxies
         self._verify = verify
+        self.timeout = None
 
     def _requests(self, method, data={}, json={}, params={}):
         if method not in ['post', 'get']:
@@ -32,7 +35,8 @@ class Query(object):
             json=json,
             params=params,
             proxies=self._proxies,
-            verify=self._verify
+            verify=self._verify,
+            timeout=self.timeout
         )
 
     def _get_query(self, type, name):
@@ -49,6 +53,7 @@ class Query(object):
         json_response = json.loads(response_text)
         if json_response.get("errors"):
             raise Exception(json_response["errors"][0]["message"])
+
         response = json_response.get("data").get(operation_name)
         if isinstance(response, dict) and response.get("error"):
             raise Exception(response["error"])
@@ -70,7 +75,7 @@ class Query(object):
                 "query": self._get_query(type='mutation', name=operation_name),
                 "variables": {
                     "data": {
-                        "username": username, 
+                        "username": username,
                         "password": password
                     }
                 },
@@ -125,31 +130,45 @@ class Query(object):
         return self._parse_response(response.text, 'getSchema')
 
     # GetGraph
-    def get_graph(self, project_id, table_id=None, filters=[], offset=0, limit=50):
+    def get_graph(self, project_id, table_id=None, filters=[], offset=0,
+                  limit=50, sort_column=None, sort_direction=None):
+
         if not project_id:
             raise Exception('Project ID required')
+
         operation_name = 'GetGraph'
         if table_id:
+            pagination = {
+                "label": table_id,
+                "offset": offset,
+                "limit": limit,
+            }
+
+            if sort_column and sort_direction:
+                pagination["sortColumn"] = sort_column
+                pagination["sortDirection"] = sort_direction
+
+            query = {
+                "startingVertices": [],
+                "labels": [table_id],
+                "pagination": pagination,
+                "filters": [dict({"label": table_id}, **filter) for filter in filters],
+                "derivedColumns": []
+            }
+
+            payload = {
+                "query": self._get_query(type='query', name=operation_name),
+                "variables": {
+                    "context": {"contextId": project_id},
+                    "contextId": project_id,
+                    "query": query
+                },
+                "operationName": operation_name
+            }
+
             response = self._requests(
                 method='post',
-                json={
-                    "query": self._get_query(type='query', name=operation_name),
-                    "variables": {
-                        "context": {"contextId": project_id},
-                        "contextId": project_id,
-                        "query": {
-                            "startingVertices": [], 
-                            "labels": [table_id], 
-                            "pagination": {
-                                "label": table_id, 
-                                "offset": offset, 
-                                "limit": limit
-                            }, 
-                            "filters": [dict({"label": table_id}, **filter) for filter in filters],
-                        }
-                    },
-                    "operationName": operation_name
-                }
+                json=payload
             )
         else:
             response = self._requests(
@@ -204,7 +223,7 @@ class Query(object):
             }
         )
         return self._parse_response(response.text, 'summarizeGraph')
-    
+
     # GetTask
     def get_task(self, task_id):
         if not task_id:
@@ -365,7 +384,7 @@ class Query(object):
             json={
                 "query": self._get_query(type='mutation', name=operation_name),
                 "variables": {
-                    "context": {"contextId": project_id}, 
+                    "context": {"contextId": project_id},
                     "data": {
                         "displayName": display_name,
                         "config": {
@@ -393,7 +412,7 @@ class Query(object):
             json={
                 "query": self._get_query(type='mutation', name=operation_name),
                 "variables": {
-                    "context": {"contextId": project_id}, 
+                    "context": {"contextId": project_id},
                     "tableId": table_id,
                     "data": {
                         "displayName": display_name,
@@ -421,7 +440,7 @@ class Query(object):
             json={
                 "query": self._get_query(type='mutation', name=operation_name),
                 "variables": {
-                    "context": {"contextId": project_id}, 
+                    "context": {"contextId": project_id},
                     "tableId": table_id,
                     "preserveRelationshipsData": preserve_relationships_data
                 },
@@ -432,7 +451,7 @@ class Query(object):
         return self._parse_response(response.text, 'deleteTable')
 
     # CreateColumn
-    def create_column(self, project_id, table_id, display_name, data_type, linked_table=None):
+    def create_column(self, project_id, table_id, display_name, data_type, linked_table=None, is_required=False):
         if not project_id:
             raise Exception('Project ID required')
         if not table_id:
@@ -449,15 +468,17 @@ class Query(object):
             json={
                 "query": self._get_query(type='mutation', name=operation_name),
                 "variables": {
-                    "context": {"contextId": project_id}, 
+                    "context": {"contextId": project_id},
                     "tableId": table_id,
                     "data": {
-                        "displayName": display_name, 
-                        "dataType": data_type, 
-                        "linkedTable": linked_table
+                        "displayName": display_name,
+                        "dataType": data_type,
+                        "linkedTable": linked_table,
+                        "isRequired": is_required
                     } if data_type in ['link_one', 'link_many'] else {
-                        "displayName": display_name, 
-                        "dataType": data_type
+                        "displayName": display_name,
+                        "dataType": data_type,
+                        "isRequired": is_required
                     }
                 },
                 "operationName": operation_name
@@ -486,7 +507,7 @@ class Query(object):
             json={
                 "query": self._get_query(type='mutation', name=operation_name),
                 "variables": {
-                    "context": {"contextId": project_id}, 
+                    "context": {"contextId": project_id},
                     "tableId": table_id,
                     "columnId": column_id,
                     "data": {
@@ -515,9 +536,9 @@ class Query(object):
             json={
                 "query": self._get_query(type='mutation', name=operation_name),
                 "variables": {
-                    "context": {"contextId": project_id}, 
+                    "context": {"contextId": project_id},
                     "tableId": table_id,
-                    "columnId": column_id,                
+                    "columnId": column_id,
                 },
                 "operationName": operation_name
             }
@@ -560,7 +581,7 @@ class Query(object):
             json={
                 "query": self._get_query(type='mutation', name=operation_name),
                 "variables": {
-                    "context": {"contextId": project_id}, 
+                    "context": {"contextId": project_id},
                     "label": table_id,
                     "data": {
                         "values": data_values,
@@ -610,7 +631,7 @@ class Query(object):
             json={
                 "query": self._get_query(type='mutation', name=operation_name),
                 "variables": {
-                    "context": {"contextId": project_id}, 
+                    "context": {"contextId": project_id},
                     "label": table_id,
                     "vertexId": vertex_id,
                     "data": {
@@ -623,7 +644,7 @@ class Query(object):
         )
         self._validate_response(response.text, 'updateVertex')
         return self._parse_response(response.text, 'updateVertex')
-    
+
     # DeleteVertex
     def delete_vertex(self, project_id, table_id, vertex_id):
         if not project_id:
@@ -638,7 +659,7 @@ class Query(object):
             json={
                 "query": self._get_query(type='mutation', name=operation_name),
                 "variables": {
-                    "context": {"contextId": project_id}, 
+                    "context": {"contextId": project_id},
                     "label": table_id,
                     "vertexId": vertex_id
                 },
@@ -662,7 +683,7 @@ class Query(object):
             json={
                 "query": self._get_query(type='mutation', name=operation_name),
                 "variables": {
-                    "context": {"contextId": project_id}, 
+                    "context": {"contextId": project_id},
                     "label": table_id,
                     "vertexIds": vertex_ids,
                     "deleteAll": False
@@ -674,17 +695,22 @@ class Query(object):
         return self._parse_response(response.text, 'bulkDeleteVertices')
 
     # Bulk Upload
-    def bulk_upload(self, project_id, filepaths):
+    def bulk_upload(self, project_id, table_id, filepath, column_ids, configs={}):
         if not project_id:
             raise Exception('Project ID required')
-        if not filepaths:
+        if not filepath:
             raise Exception('CSV Files required')
 
+        csv_file = open(filepath)
+        csv_reader = csv.reader(csv_file)
+        for row in csv_reader:
+            if len(row) != len(column_ids):
+                raise Exception('Number of column ids not matched with csv file')
+            break
+
         # Upload
         multipart_form_data = []
-        for filepath in filepaths:
-            multipart_form_data.append(
-                ('files', (filepath.split('/')[-1], open(filepath, 'rb'))))
+        multipart_form_data.append(('files', (filepath.split('/')[-1], open(filepath, 'rb'))))
         multipart_form_data = tuple(multipart_form_data)
         response = self._session.post(
             self.BULK_UPLOAD_URL,
@@ -692,40 +718,56 @@ class Query(object):
             data={
                 "organization_id": self._organization_id,
                 "project_id": project_id,
-                "file_type": "csv"
+                "file_type": "single_csv"
             }
         )
         if response.status_code != 200:
-            raise Exception('Something went wrong')
+            raise Exception('Something went wrong in uploading csv file')
         bundle_id = json.loads(response.content)['bundle_id']
+        if not bundle_id:
+            raise Exception('Bundle id not found')
 
-        operation_name = 'BulkStartProcessing'
+        operation_name = 'BulkStartImport'
         response = self._requests(
             method='post',
             json={
                 "query": self._get_query(type='mutation', name=operation_name),
                 "variables": {
                     "bundleId": bundle_id,
+                    "context": {"contextId": project_id},
+                    "sheetConfigs": [{
+                        "columnMap": [{"csvColumnIndex": i, "mappedColumnId": column_id, "wantsCreate": False} for i, column_id in enumerate(column_ids) if column_id is not None],
+                        "countSkipRows": configs.get("countSkipRows", 0),
+                        "dropEmpty": configs.get("dropEmpty", False),
+                        "hasHeader": configs.get("hasHeader", False),
+                        "sheetName": "default",
+                        "targetTable": table_id
+                    }]
                 },
                 "operationName": operation_name
             }
         )
-        self._validate_response(response.text, 'bulkStartProcessing')
+        self._validate_response(response.text, 'bulkStartImport')
+        task_id = json.loads(response.text).get('data', {}).get('bulkStartImport', {}).get('taskId')
+        if not task_id:
+            raise Exception('Task id not found')
 
-        status = 'running'
+        status = 'queued'
         while status != 'finished' and status != 'failed':
-            operation_name = 'GetBulkBundle'
+            operation_name = 'GetTask'
             response = self._requests(
                 method='post',
                 json={
                     "query": self._get_query(type='query', name=operation_name),
                     "variables": {
-                        "bundleId": bundle_id,
+                        "taskId": task_id,
                     },
                     "operationName": operation_name
                 }
             )
-            status = json.loads(response.text).get('data', {}).get('getBulkBundle', {}).get('status')
-            time.sleep(5)
-        self._validate_response(response.text, 'getBulkBundle')
-        return self._parse_response(response.text, 'getBulkBundle')
+            self._validate_response(response.text, 'getTask')
+            status = json.loads(response.text).get('data', {}).get('getTask', {}).get('status')
+            progress = json.loads(response.text).get('data', {}).get('getTask', {}).get('progress')
+            print ("progress", progress)
+            time.sleep(2)
+        return self._parse_response(response.text, 'getTask')
